@@ -12,7 +12,6 @@ import (
 	"errors"
 	"fmt"
 	"io"
-	"io/ioutil"
 	"os"
 	"os/exec"
 	"path/filepath"
@@ -56,31 +55,12 @@ GRUB_TERMINAL=console
 GRUB_RECORDFAIL_TIMEOUT=0
 `
 
-func (img *CoreGrubImage) Mount() (err error) {
-	img.baseMount, err = ioutil.TempDir(os.TempDir(), "core-grub-disk")
+func (img *CoreGrubImage) Mount() error {
+	baseMount, err := mount(img.parts)
 	if err != nil {
-		return errors.New(fmt.Sprintf("Unable to create temp dir to create system image: %s", err))
-	}
-	//Remove Mountpoint if we fail along the way
-	defer func() {
-		if err != nil {
-			os.Remove(img.baseMount)
-		}
-	}()
-
-	for _, part := range img.parts {
-		if part.fs == fsNone {
-			continue
-		}
-
-		mountpoint := filepath.Join(img.baseMount, string(part.dir))
-		if err := os.MkdirAll(mountpoint, 0755); err != nil {
-			return err
-		}
-		if out, err := exec.Command("mount", filepath.Join("/dev/mapper", part.loop), mountpoint).CombinedOutput(); err != nil {
-			return fmt.Errorf("unable to mount dir to create system image: %s", out)
-		}
+		return err
 	}
+	img.baseMount = baseMount
 
 	return nil
 }
@@ -281,7 +261,7 @@ func (img CoreGrubImage) BaseMount() string {
 	return img.baseMount
 }
 
-func (img *CoreGrubImage) SetupBoot(oem OemDescription) error {
+func (img *CoreGrubImage) SetupBoot(oemRootPath string) error {
 	for _, dev := range []string{"dev", "proc", "sys"} {
 		src := filepath.Join("/", dev)
 		dst := filepath.Join(img.System(), dev)
@@ -351,7 +331,7 @@ func (img *CoreGrubImage) SetupBoot(oem OemDescription) error {
 	}
 	grubFile, err := os.Create(filepath.Join(grubDir, "50-system-image.cfg"))
 	if err != nil {
-		return fmt.Errorf("unable to create %s file: %s", grubFile, err)
+		return fmt.Errorf("unable to create %s file: %s", grubFile.Name(), err)
 	}
 	defer grubFile.Close()
 	if _, err := io.WriteString(grubFile, grubCfgContent); err != nil {
@@ -368,7 +348,7 @@ func (img *CoreGrubImage) SetupBoot(oem OemDescription) error {
 	return nil
 }
 
-func (img *CoreGrubImage) FlashExtra(devicePart string) error {
+func (img *CoreGrubImage) FlashExtra(oemRootPath, devicePart string) error {
 	return nil
 }
 
