@@ -11,7 +11,6 @@ import (
 	"bufio"
 	"errors"
 	"fmt"
-	"io"
 	"io/ioutil"
 	"os"
 	"os/exec"
@@ -20,7 +19,6 @@ import (
 	"text/template"
 
 	"launchpad.net/goget-ubuntu-touch/sysutils"
-	"launchpad.net/goyaml"
 )
 
 // This program is free software: you can redistribute it and/or modify it
@@ -39,11 +37,11 @@ type CoreUBootImage struct {
 	CoreImage
 	SystemImage
 	hardware  HardwareDescription
+	oem       OemDescription
 	location  string
 	size      int64
 	baseMount string
 	parts     []partition
-	platform  string
 }
 
 const snappySystemTemplate = `# This is a snappy variables and boot logic file and is entirely generated and
@@ -79,36 +77,21 @@ type FlashInstructions struct {
 	Bootloader []string `yaml:"bootloader"`
 }
 
-func NewCoreUBootImage(location string, size int64, hw HardwareDescription, platform string) *CoreUBootImage {
+func NewCoreUBootImage(location string, size int64, hw HardwareDescription, oem OemDescription) *CoreUBootImage {
 	return &CoreUBootImage{
 		hardware: hw,
+		oem:      oem,
 		location: location,
 		size:     size,
-		platform: platform,
 	}
 }
 
-func (img *CoreUBootImage) Mount() (err error) {
-	img.baseMount, err = ioutil.TempDir(os.TempDir(), "core-uboot-disk")
+func (img *CoreUBootImage) Mount() error {
+	baseMount, err := mount(img.parts)
 	if err != nil {
-		return errors.New(fmt.Sprintf("Unable to create temp dir to create system image: %s", err))
-	}
-	//Remove Mountpoint if we fail along the way
-	defer func() {
-		if err != nil {
-			os.Remove(img.baseMount)
-		}
-	}()
-
-	for _, part := range img.parts {
-		mountpoint := filepath.Join(img.baseMount, string(part.dir))
-		if err := os.MkdirAll(mountpoint, 0755); err != nil {
-			return err
-		}
-		if out, err := exec.Command("mount", filepath.Join("/dev/mapper", part.loop), mountpoint).CombinedOutput(); err != nil {
-			return fmt.Errorf("unable to mount dir to create system image: %s", out)
-		}
+		return err
 	}
+	img.baseMount = baseMount
 
 	return nil
 }
@@ -290,7 +273,7 @@ func (img CoreUBootImage) BaseMount() string {
 	return img.baseMount
 }
 
-func (img CoreUBootImage) SetupBoot(oem OemDescription) error {
+func (img CoreUBootImage) SetupBoot(oemRootPath string) error {
 	// destinations
 	bootPath := filepath.Join(img.baseMount, string(bootDir))
 	bootAPath := filepath.Join(bootPath, "a")
@@ -313,15 +296,15 @@ func (img CoreUBootImage) SetupBoot(oem OemDescription) error {
 			return err
 		}
 
-		if err := copyFile(hardwareYamlPath, filepath.Join(path, "hardware.yaml")); err != nil {
+		if err := sysutils.CopyFile(hardwareYamlPath, filepath.Join(path, "hardware.yaml")); err != nil {
 			return err
 		}
 
-		if err := copyFile(kernelPath, filepath.Join(path, filepath.Base(kernelPath))); err != nil {
+		if err := sysutils.CopyFile(kernelPath, filepath.Join(path, filepath.Base(kernelPath))); err != nil {
 			return err
 		}
 
-		if err := copyFile(initrdPath, filepath.Join(path, filepath.Base(initrdPath))); err != nil {
+		if err := sysutils.CopyFile(initrdPath, filepath.Join(path, filepath.Base(initrdPath))); err != nil {
 			return err
 		}
 
@@ -331,13 +314,21 @@ func (img CoreUBootImage) SetupBoot(oem OemDescription) error {
 			return err
 		}
 
-		if err := img.provisionDtbs(oem, bootDtbPath); err != nil {
+		if err := img.provisionDtbs(bootDtbPath); err != nil {
 			return err
 		}
 	}
 
-	if err := img.provisionUenv(bootuEnvPath); err != nil {
-		return err
+	// if the oem package provides BootAssets use it directly, if not
+	// provisionUenv for backwards compatibility.
+	if bootAssets := img.oem.OEM.Hardware.BootAssets; bootAssets != nil {
+		if err := setupBootAssetFiles(bootPath, oemRootPath, bootAssets.Files); err != nil {
+			return err
+		}
+	} else {
+		if err := img.provisionUenv(bootuEnvPath); err != nil {
+			return err
+		}
 	}
 
 	// create /boot/uboot
@@ -352,8 +343,8 @@ func (img CoreUBootImage) SetupBoot(oem OemDescription) error {
 	defer snappySystemFile.Close()
 
 	var fdtfile string
-	if img.platform != "" {
-		fdtfile = fmt.Sprintf("fdtfile=%s.dtb", img.platform)
+	if platform := img.oem.Platform(); platform != "" {
+		fdtfile = fmt.Sprintf("fdtfile=%s.dtb", platform)
 	}
 
 	t := template.Must(template.New("snappy-system").Parse(snappySystemTemplate))
@@ -363,12 +354,14 @@ func (img CoreUBootImage) SetupBoot(oem OemDescription) error {
 }
 
 func (img CoreUBootImage) provisionUenv(bootuEnvPath string) error {
-	if img.platform == "" {
+	platform := img.oem.Platform()
+
+	if platform != "" {
 		printOut("No platform select, not searching for uEnv.txt")
 		return nil
 	}
 
-	flashAssetsPath := filepath.Join(img.baseMount, "flashtool-assets", img.platform)
+	flashAssetsPath := filepath.Join(img.baseMount, "flashtool-assets", platform)
 	uEnvPath := filepath.Join(flashAssetsPath, "uEnv.txt")
 
 	if _, err := os.Stat(flashAssetsPath); os.IsNotExist(err) {
@@ -381,7 +374,7 @@ func (img CoreUBootImage) provisionUenv(bootuEnvPath string) error {
 	// if a uEnv.txt is provided in the flashtool-assets, use it
 	if _, err := os.Stat(uEnvPath); err == nil {
 		printOut("Adding uEnv.txt to", bootuEnvPath)
-		if err := copyFile(uEnvPath, bootuEnvPath); err != nil {
+		if err := sysutils.CopyFile(uEnvPath, bootuEnvPath); err != nil {
 			return err
 		}
 	} else {
@@ -391,7 +384,7 @@ func (img CoreUBootImage) provisionUenv(bootuEnvPath string) error {
 	return nil
 }
 
-func (img CoreUBootImage) provisionDtbs(oem OemDescription, bootDtbPath string) error {
+func (img CoreUBootImage) provisionDtbs(bootDtbPath string) error {
 	dtbsPath := filepath.Join(img.baseMount, img.hardware.Dtbs)
 
 	if _, err := os.Stat(dtbsPath); os.IsNotExist(err) {
@@ -405,26 +398,31 @@ func (img CoreUBootImage) provisionDtbs(oem OemDescription, bootDtbPath string)
 		return err
 	}
 
-	dtb := filepath.Join(dtbsPath, fmt.Sprintf("%s.dtb", img.platform))
+	dtb := filepath.Join(dtbsPath, fmt.Sprintf("%s.dtb", img.oem.Platform()))
 
 	// if there is a specific dtb for the platform, copy it.
 	// First look in oem and then in device.
-	if oem.Hardware.Dtb != "" && img.platform != "" {
-		oemDtb := filepath.Join(img.System(), oem.InstallPath(), oem.Hardware.Dtb)
+	if oemDtb := img.oem.OEM.Hardware.Dtb; oemDtb != "" && img.oem.Platform() != "" {
+		oemInstallPath, err := img.oem.InstallPath(img.System())
+		if err != nil {
+			return err
+		}
+
+		oemDtb := filepath.Join(oemInstallPath, oemDtb)
 		dst := filepath.Join(bootDtbPath, filepath.Base(dtb))
-		if err := copyFile(oemDtb, dst); err != nil {
+		if err := sysutils.CopyFile(oemDtb, dst); err != nil {
 			return err
 		}
 	} else if _, err := os.Stat(dtb); err == nil {
 		dst := filepath.Join(bootDtbPath, filepath.Base(dtb))
-		if err := copyFile(dtb, dst); err != nil {
+		if err := sysutils.CopyFile(dtb, dst); err != nil {
 			return err
 		}
 	} else {
 		for _, dtbFi := range dtbFis {
 			src := filepath.Join(dtbsPath, dtbFi.Name())
 			dst := filepath.Join(bootDtbPath, dtbFi.Name())
-			if err := copyFile(src, dst); err != nil {
+			if err := sysutils.CopyFile(src, dst); err != nil {
 				return err
 			}
 		}
@@ -433,85 +431,12 @@ func (img CoreUBootImage) provisionDtbs(oem OemDescription, bootDtbPath string)
 	return nil
 }
 
-func (img *CoreUBootImage) FlashExtra(devicePart string) error {
-	if img.platform == "" {
-		return nil
-	}
-
-	tmpdir, err := ioutil.TempDir("", "device")
-	if err != nil {
-		return errors.New("cannot create tempdir to extract hardware.yaml from device part")
-	}
-	defer os.RemoveAll(tmpdir)
-
-	if out, err := exec.Command("tar", "xf", devicePart, "-C", tmpdir, "flashtool-assets").CombinedOutput(); err != nil {
-		fmt.Println("No flashtool-assets found, skipping...")
-		if debugPrint {
-			fmt.Println("device part:", devicePart)
-			fmt.Println("command output:", string(out))
-		}
-
-		return nil
-	}
-
-	flashAssetsPath := filepath.Join(tmpdir, "flashtool-assets", img.platform)
-	flashPath := filepath.Join(flashAssetsPath, "flash.yaml")
-
-	if _, err := os.Stat(flashPath); err != nil && os.IsNotExist(err) {
-		return nil
-	} else if err != nil {
-		return err
-	}
-
-	data, err := ioutil.ReadFile(flashPath)
-	if err != nil {
-		return err
-	}
-
-	var flash FlashInstructions
-	if err := goyaml.Unmarshal([]byte(data), &flash); err != nil {
-		return err
-	}
-
-	fmt.Println("Running flashtool-asset commands")
-	for _, cmd := range flash.Bootloader {
-		cmd = fmt.Sprintf(cmd, flashAssetsPath, img.location)
-		cmdFields := strings.Fields(cmd)
-
-		if out, err := exec.Command(cmdFields[0], cmdFields[1:]...).CombinedOutput(); err != nil {
-			return fmt.Errorf("failed to run flash command: %s : %s", cmd, out)
-		} else {
-			fmt.Println(cmd)
-			fmt.Println(string(out))
-		}
+func (img *CoreUBootImage) FlashExtra(oemRootPath, devicePart string) error {
+	// if the oem package has bootloader assets use this and skip the
+	// deprecated device part setup
+	if bootAssets := img.oem.OEM.Hardware.BootAssets; bootAssets != nil {
+		return setupBootAssetRawFiles(img.location, oemRootPath, bootAssets.RawFiles)
 	}
 
 	return nil
 }
-
-func copyFile(src, dst string) error {
-	dstFile, err := os.Create(dst)
-	if err != nil {
-		return err
-	}
-	defer dstFile.Close()
-
-	srcFile, err := os.Open(src)
-	if err != nil {
-		return err
-	}
-	defer srcFile.Close()
-
-	reader := bufio.NewReader(srcFile)
-	writer := bufio.NewWriter(dstFile)
-	defer func() {
-		if err != nil {
-			writer.Flush()
-		}
-	}()
-	if _, err = io.Copy(writer, reader); err != nil {
-		return err
-	}
-	writer.Flush()
-	return nil
-}
