@@ -22,7 +22,11 @@ import (
 	"syscall"
 	"time"
 
-	"launchpad.net/goyaml"
+	"gopkg.in/yaml.v2"
+	"launchpad.net/snappy/helpers"
+	"launchpad.net/snappy/progress"
+	"launchpad.net/snappy/release"
+	"launchpad.net/snappy/snappy"
 
 	"launchpad.net/goget-ubuntu-touch/diskimage"
 	"launchpad.net/goget-ubuntu-touch/sysutils"
@@ -49,22 +53,30 @@ func init() {
 }
 
 type CoreCmd struct {
-	Channel       string   `long:"channel" description:"Specify the channel to use" default:"ubuntu-core/devel"`
-	Device        string   `long:"device" description:"Specify the device to use" default:"generic_amd64"`
-	Keyboard      string   `long:"keyboard-layout" description:"Specify the keyboard layout" default:"us"`
-	Output        string   `long:"output" short:"o" description:"Name of the image file to create" required:"true"`
-	Size          int64    `long:"size" short:"s" description:"Size of image file to create in GB (min 4)" default:"20"`
-	DeveloperMode bool     `long:"developer-mode" description:"Finds the latest public key in your ~/.ssh and sets it up using cloud-init"`
-	EnableSsh     bool     `long:"enable-ssh" description:"Enable ssh on the image through cloud-init(not needed with developer mode)"`
-	Cloud         bool     `long:"cloud" description:"Generate a pure cloud image without setting up cloud-init"`
-	Platform      string   `long:"platform" description:"specify the boards platform"`
-	Install       []string `long:"install" description:"install additional packages (can be called multiple times)"`
+	Channel string `long:"channel" description:"Specify the channel to use" default:"stable"`
+	Output  string `long:"output" short:"o" description:"Name of the image file to create" required:"true"`
+	Size    int64  `long:"size" short:"s" description:"Size of image file to create in GB (min 4)" default:"4"`
+	Oem     string `long:"oem" description:"The snappy oem package to base the image out of" default:"generic-amd64"`
 
 	Development struct {
-		DevicePart string `long:"device-part" description:"Specify a local device part to override the one from the server"`
+		DevicePart    string `long:"device-part" description:"Specify a local device part to override the one from the server"`
+		DeveloperMode bool   `long:"developer-mode" description:"Finds the latest public key in your ~/.ssh and sets it up using cloud-init"`
+		EnableSsh     bool   `long:"enable-ssh" description:"Enable ssh on the image through cloud-init(not needed with developer mode)"`
 	} `group:"Development"`
 
-	hardware diskimage.HardwareDescription
+	Deprecated struct {
+		Install []string `long:"install" description:"Install additional packages (can be called multiple times)"`
+		Cloud   bool     `long:"cloud" description:"Generate a pure cloud image without setting up cloud-init"`
+		Device  string   `long:"device" description:"Specify the device to use"`
+	} `group:"Deprecated"`
+
+	Positional struct {
+		Release string `positional-arg-name:"release" description:"The release to base the image out of (15.04 or rolling)" required:"true"`
+	} `positional-args:"yes" required:"yes"`
+
+	hardware        diskimage.HardwareDescription
+	oem             diskimage.OemDescription
+	stagingRootPath string
 }
 
 var coreCmd CoreCmd
@@ -80,7 +92,12 @@ ssh_genkeytypes: ['rsa', 'dsa', 'ecdsa', 'ed25519']
 `
 
 func (coreCmd *CoreCmd) Execute(args []string) error {
-	if coreCmd.EnableSsh && coreCmd.Cloud {
+	// we don't want to overwrite the output, people might get angry :-)
+	if helpers.FileExists(coreCmd.Output) {
+		return fmt.Errorf("Giving up, the desired target output file %#v already exists", coreCmd.Output)
+	}
+
+	if coreCmd.Development.EnableSsh && coreCmd.Deprecated.Cloud {
 		return errors.New("--cloud and --enable-ssh cannot be used together")
 	}
 
@@ -94,6 +111,12 @@ func (coreCmd *CoreCmd) Execute(args []string) error {
 		devicePart = p
 	}
 
+	fmt.Println("Determining oem configuration")
+	if err := coreCmd.extractOem(coreCmd.Oem); err != nil {
+		return err
+	}
+	defer os.RemoveAll(coreCmd.stagingRootPath)
+
 	if !globalArgs.DownloadOnly {
 		if syscall.Getuid() != 0 {
 			return errors.New("command requires sudo/pkexec (root)")
@@ -108,13 +131,22 @@ func (coreCmd *CoreCmd) Execute(args []string) error {
 	}
 
 	fmt.Println("Fetching information from server...")
-
 	channels, err := ubuntuimage.NewChannels(globalArgs.Server)
 	if err != nil {
 		return err
 	}
 
-	deviceChannel, err := channels.GetDeviceChannel(globalArgs.Server, coreCmd.Channel, coreCmd.Device)
+	channel := systemImageChannel("ubuntu-core", coreCmd.Positional.Release, coreCmd.Channel)
+	// TODO: remove once azure channel is gone
+	var device string
+	if coreCmd.Deprecated.Device != "" {
+		fmt.Println("WARNING: this option should only be used to build azure images")
+		device = coreCmd.Deprecated.Device
+	} else {
+		device = systemImageDeviceChannel(coreCmd.oem.Architecture())
+	}
+
+	deviceChannel, err := channels.GetDeviceChannel(globalArgs.Server, channel, device)
 	if err != nil {
 		return err
 	}
@@ -185,7 +217,7 @@ func (coreCmd *CoreCmd) Execute(args []string) error {
 			f := <-filePathChan
 			baseFile := filepath.Base(f)
 
-			if err := copyFile(f, filepath.Join(workDir, baseFile)); err != nil {
+			if err := sysutils.CopyFile(f, filepath.Join(workDir, baseFile)); err != nil {
 				return err
 			}
 			downloadedFiles = append(downloadedFiles, baseFile)
@@ -200,13 +232,14 @@ func (coreCmd *CoreCmd) Execute(args []string) error {
 		return nil
 	}
 
-	switch coreCmd.hardware.Bootloader {
+	loader := coreCmd.oem.OEM.Hardware.Bootloader
+	switch loader {
 	case "grub":
 		img = diskimage.NewCoreGrubImage(coreCmd.Output, coreCmd.Size)
 	case "u-boot":
-		img = diskimage.NewCoreUBootImage(coreCmd.Output, coreCmd.Size, coreCmd.hardware, coreCmd.Platform)
+		img = diskimage.NewCoreUBootImage(coreCmd.Output, coreCmd.Size, coreCmd.hardware, coreCmd.oem)
 	default:
-		fmt.Printf("Bootloader set to '%s' in hardware description, assuming grub as a fallback\n", coreCmd.hardware.Bootloader)
+		fmt.Printf("Bootloader set to '%s' in oem hardware description, assuming grub as a fallback\n", loader)
 		img = diskimage.NewCoreGrubImage(coreCmd.Output, coreCmd.Size)
 	}
 
@@ -254,12 +287,23 @@ func (coreCmd *CoreCmd) Execute(args []string) error {
 		return err
 	}
 
-	if err := img.FlashExtra(devicePart); err != nil {
+	oemRootPath, err := coreCmd.oem.InstallPath(coreCmd.stagingRootPath)
+	if coreCmd.Oem != "" && err != nil {
+		return err
+	}
+
+	if err := img.FlashExtra(oemRootPath, devicePart); err != nil {
 		return err
 	}
 
 	fmt.Println("New image complete")
-	if coreCmd.Device != "generic_armhf" {
+	fmt.Println("Summary:")
+	fmt.Println(" Output:", coreCmd.Output)
+	fmt.Println(" Architecture:", coreCmd.oem.Architecture())
+	fmt.Println(" Channel:", coreCmd.Channel)
+	fmt.Println(" Version:", image.Version)
+
+	if coreCmd.oem.Architecture() != "armhf" {
 		fmt.Println("Launch by running: kvm -m 768", coreCmd.Output)
 	}
 
@@ -323,21 +367,16 @@ func (coreCmd *CoreCmd) setup(img diskimage.CoreImage, filePathChan <-chan strin
 		return err
 	}
 
-	// check if we installed an oem snap
-	oem, err := loadOem(systemPath)
-	if err != nil {
+	oemRootPath, err := coreCmd.oem.InstallPath(coreCmd.stagingRootPath)
+	if coreCmd.Oem != "" && err != nil {
 		return err
 	}
 
-	if err := img.SetupBoot(oem); err != nil {
+	if err := img.SetupBoot(oemRootPath); err != nil {
 		return err
 	}
 
-	if err := coreCmd.setupKeyboardLayout(systemPath); err != nil {
-		return err
-	}
-
-	if !coreCmd.Cloud {
+	if !coreCmd.Deprecated.Cloud {
 		cloudBaseDir := filepath.Join("var", "lib", "cloud")
 
 		if err := os.MkdirAll(filepath.Join(systemPath, cloudBaseDir), 0755); err != nil {
@@ -351,7 +390,7 @@ func (coreCmd *CoreCmd) setup(img diskimage.CoreImage, filePathChan <-chan strin
 
 	// if the device is armhf, we can't to make this copy here since it's faster
 	// than on the device.
-	if coreCmd.Device == "generic_armhf" && coreCmd.hardware.PartitionLayout == "system-AB" {
+	if coreCmd.oem.Architecture() == archArmhf && coreCmd.oem.PartitionLayout() == "system-AB" {
 		printOut("Replicating system-a into system-b")
 
 		src := fmt.Sprintf("%s/.", systemPath)
@@ -367,28 +406,29 @@ func (coreCmd *CoreCmd) setup(img diskimage.CoreImage, filePathChan <-chan strin
 }
 
 func (coreCmd *CoreCmd) install(systemPath string) error {
-	if strings.Contains(coreCmd.Device, "armhf") {
-		if err := sysutils.AddQemuStatic(systemPath); err != nil {
-			return err
-		}
-		defer sysutils.RemoveQemuStatic(systemPath)
+	snappy.SetRootDir(systemPath)
+	defer snappy.SetRootDir("/")
+
+	flags := coreCmd.installFlags()
+	oemSoftware := coreCmd.oem.OEM.Software
+	packageCount := len(coreCmd.Deprecated.Install) + len(oemSoftware.BuiltIn) + len(oemSoftware.Preinstalled)
+	if coreCmd.Oem != "" {
+		packageCount += 1
 	}
+	packageQueue := make([]string, 0, packageCount)
 
-	if err := sysutils.ChrootBindMount(systemPath); err != nil {
-		return err
+	if coreCmd.Oem != "" {
+		packageQueue = append(packageQueue, coreCmd.Oem)
 	}
-	defer sysutils.ChrootBindUnmount(systemPath)
+	packageQueue = append(packageQueue, oemSoftware.BuiltIn...)
+	packageQueue = append(packageQueue, oemSoftware.Preinstalled...)
+	packageQueue = append(packageQueue, coreCmd.Deprecated.Install...)
 
-	for _, snap := range coreCmd.Install {
-		snapBase := filepath.Base(snap)
-		fmt.Println("Installing", snapBase)
+	for _, snap := range packageQueue {
+		fmt.Println("Installing", snap)
 
-		if err := copyFile(snap, filepath.Join(systemPath, snapBase)); err != nil {
-			return err
-		}
-		defer os.Remove(filepath.Join(systemPath, snapBase))
-
-		if err := sysutils.ChrootRun(systemPath, "snappy", "install", snapBase); err != nil {
+		pb := progress.NewTextProgress(snap)
+		if _, err := snappy.Install(snap, flags, pb); err != nil {
 			return err
 		}
 	}
@@ -413,7 +453,7 @@ func (coreCmd *CoreCmd) setupCloudInit(cloudBaseDir, systemData string) error {
 		return err
 	}
 
-	if coreCmd.DeveloperMode {
+	if coreCmd.Development.DeveloperMode {
 		fmt.Println("Enabling developer mode...")
 
 		authorizedKey, err := getAuthorizedSshKey()
@@ -441,7 +481,7 @@ func (coreCmd *CoreCmd) setupCloudInit(cloudBaseDir, systemData string) error {
 		return err
 	}
 
-	if coreCmd.DeveloperMode || coreCmd.EnableSsh {
+	if coreCmd.Development.DeveloperMode || coreCmd.Development.EnableSsh {
 		if _, err := io.WriteString(userDataFile, "snappy:\n    ssh_enabled: True\n"); err != nil {
 			return err
 		}
@@ -450,25 +490,6 @@ func (coreCmd *CoreCmd) setupCloudInit(cloudBaseDir, systemData string) error {
 	return nil
 }
 
-func (coreCmd *CoreCmd) setupKeyboardLayout(systemPath string) error {
-	kbFilePath := filepath.Join(systemPath, "etc", "default", "keyboard")
-
-	// do not error if the image has no keyboard
-	if _, err := os.Stat(kbFilePath); err != nil && os.IsNotExist(err) {
-		return nil
-	}
-
-	kbFileContents, err := ioutil.ReadFile(kbFilePath)
-	if err != nil {
-		return err
-	}
-
-	r := strings.NewReplacer("XKBLAYOUT=\"us\"", fmt.Sprintf("XKBLAYOUT=\"%s\"", coreCmd.Keyboard))
-	kbFileContents = []byte(r.Replace(string(kbFileContents)))
-
-	return ioutil.WriteFile(kbFilePath, kbFileContents, 0644)
-}
-
 func getAuthorizedSshKey() (string, error) {
 	sshDir := os.ExpandEnv("$HOME/.ssh")
 
@@ -500,7 +521,57 @@ func getAuthorizedSshKey() (string, error) {
 	return string(pubKey), err
 }
 
-func loadOem(systemPath string) (oem diskimage.OemDescription, err error) {
+func (coreCmd *CoreCmd) installFlags() snappy.InstallFlags {
+	flags := snappy.InhibitHooks | snappy.AllowOEM
+
+	if coreCmd.Development.DeveloperMode {
+		flags |= snappy.AllowUnauthenticated
+	}
+
+	return flags
+}
+
+func (coreCmd *CoreCmd) extractOem(oemPackage string) error {
+	if oemPackage == "" {
+		return nil
+	}
+
+	tempDir, err := ioutil.TempDir("", "oem")
+	if err != nil {
+		return err
+	}
+
+	// we need to fix the permissions for tempdir to  be seteuid friendly
+	if err := os.Chmod(tempDir, 0755); err != nil {
+		return err
+	}
+
+	coreCmd.stagingRootPath = tempDir
+
+	snappy.SetRootDir(tempDir)
+	defer snappy.SetRootDir("/")
+	release.Override(release.Release{
+		Flavor:  "core",
+		Series:  coreCmd.Positional.Release,
+		Channel: coreCmd.Channel,
+	})
+
+	flags := coreCmd.installFlags()
+	pb := progress.NewTextProgress(oemPackage)
+	if _, err := snappy.Install(oemPackage, flags, pb); err != nil {
+		return err
+	}
+
+	oem, err := coreCmd.loadOem(tempDir)
+	if err != nil {
+		return err
+	}
+	coreCmd.oem = oem
+
+	return nil
+}
+
+func (coreCmd CoreCmd) loadOem(systemPath string) (oem diskimage.OemDescription, err error) {
 	pkgs, err := filepath.Glob(filepath.Join(systemPath, "/oem/*/*/meta/package.yaml"))
 	if err != nil {
 		return oem, err
@@ -518,7 +589,7 @@ func loadOem(systemPath string) (oem diskimage.OemDescription, err error) {
 		return oem, errors.New("failed to read oem yaml")
 	}
 
-	if err := goyaml.Unmarshal([]byte(f), &oem); err != nil {
+	if err := yaml.Unmarshal([]byte(f), &oem); err != nil {
 		return oem, errors.New("cannot decode oem yaml")
 	}
 
@@ -552,7 +623,7 @@ func extractHWDescription(path string) (hw diskimage.HardwareDescription, err er
 		return hw, err
 	}
 
-	err = goyaml.Unmarshal([]byte(data), &hw)
+	err = yaml.Unmarshal([]byte(data), &hw)
 
 	return hw, err
 }
